import React, { useEffect, useState } from 'react'
import { yupResolver } from '@hookform/resolvers/yup'
import { useRecoilState } from 'recoil'
import { messageAtom } from 'src/lib/atoms/messageAtom'
import * as yup from 'yup'
import useSWR from 'swr'
import useErrors from 'src/hooks/useErrors'
import useAxios from 'src/hooks/useAxios'
import { useForm } from 'react-hook-form'
import { SaveIcon } from '@heroicons/react/solid'
import {
  Button,
  FormLabel,
  FormControlLabel,
  CircularProgress,
  Dialog,
  DialogActions,
  DialogContent,
  DialogContentText,
  DialogTitle,
  FormControl,
  FormGroup,
  FormHelperText,
  TextField
} from '@mui/material'
import Checkbox from '@mui/material/Checkbox'
import Box from '@mui/material/Box'

const RoleForm = ({ onClose, onSuccess, setOpenForm, roleId }) => {
  const [open, setOpen] = useState(true) // This state is used for Opening the Form
  const [loading, setLoading] = useState(false) // This state is used for setting Loader
  const [buttonLoading, setButtonLoading] = useState(false) // This state is used for setting the Button Click loader
  const [message, setMessage] = useRecoilState(messageAtom)
  const [permissionList, setPermissionList] = useState([])
  const { validationErrors } = useErrors() // Validating the Errors that come from the response
  const request = useAxios()
  const { errorHandler } = useErrors()

  // Once we finish the INSERT or UPDATE task we call this method to close the form
  // And change the states
  const handleClose = () => {
    setOpen(false)
    onClose(false)
    setOpenForm(false)
  }

  // The schema for the inputs that will be submitted
  const schema = yup.object().shape({
    name: yup.string().required()
  })

  // Required variables for using the useForm hook
  const {
    handleSubmit,
    register,
    formState: { errors },
    setError,
    setValue
  } = useForm({
    resolver: yupResolver(schema)
  })

  // Handling the loader if the roleId is set
  useEffect(() => {
    if (roleId !== 0) {
      setLoading(true)
    }
  }, [roleId])

  // Handling the submit functionality of the form and checking the type of the method
  // to be send to the server
  const onSubmit = data => {
    if (roleId !== 0) {
      data = { ...data, role_id: roleId, permissions: permissionList }
    }

    setButtonLoading(true)
    let url = roleId === 0 ? '/roles' : `roles/${roleId}`
    let method = roleId === 0 ? 'POST' : 'PUT'

    request({
      url: url,
      method: method,
      data: data
    })
      .then(res => {
        if (res?.data?.status) {
          setButtonLoading(false)
          setMessage(res?.data?.message)
          onSuccess(true)
          handleClose()
        }
      })
      .catch(err => {
        setButtonLoading(false)
        validationErrors(err, setError)
      })
  }

  // Retriving the data respected to the userId
  const { data } = useSWR(`get/roles/${roleId}`, () =>
    request
      .get(`/roles/${roleId}`)
      .then(res => res.data)
      .catch(err => errorHandler(err))
  )

  const { data: permissions } = useSWR(`get/permissions`, () =>
    request
      .get('/permissions')
      .then(res => res.data)
      .catch(err => errorHandler(err))
  )

  const handleChange = event => {
    var updatedList = [...permissionList]
    if (event.target.checked) {
      updatedList = [...permissionList, Number.parseInt(event.target.value)]
    } else {
      updatedList.splice(permissionList.indexOf(Number.parseInt(event.target.value)), 1)
    }
    setPermissionList(updatedList)
  }

  // Setting fields value respected to the retrieved data from the request
  // If the userId is greater then 0 and data is not empty
  useEffect(() => {
    if (roleId !== 0 && data) {
      data = data?.data
      setValue('name', data?.name)
      setPermissionList(data?.assigned_permissions)
      setLoading(false)
    }
  }, [data, roleId])

  useEffect(() => {
    setValue('permissions', permissionList)
    console.log(permissionList)
  }, [permissionList])

  // Return the render content
  return (
    <div>
      <Dialog
        open={open}
        keepMounted
        onClose={handleClose}
        aria-describedby={'alert-dialog-slide-description'}
        maxWidth={'md'}
        fullWidth={true}
      >
        <form onSubmit={handleSubmit(onSubmit)}>
          <DialogTitle>{roleId !== 0 ? 'Edit Role' : 'Add Role'}</DialogTitle>
          <DialogContent>
            <DialogContentText className='py-2' component={'div'}>
              {loading ? (
                <div className={'py-8 text-center'}>
                  <CircularProgress size={40} />
                </div>
              ) : (
                <Box
                  sx={{
                    '& .MuiTextField-root': {
                      mt: 3,
                      mb: 2,
                      pr: 3
                    },
                    '& .MuiFormControl-root': {
                      mt: 3,
                      mb: 2,
                      pr: 3
                    },
                    display: 'flex',
                    flexDirection: 'row',
                    flexWrap: 'wrap'
                  }}
                >
                  <TextField
                    id='name'
                    size='small'
                    fullWidth
                    {...register('name')}
                    label='Name'
                    error={errors['name']}
                    helperText={errors['name']?.message}
                  />
                  <FormControl sx={{ m: 3 }} component='fieldset' variant='standard'>
                    <FormLabel component='legend'>Permissions</FormLabel>
                    <FormGroup sx={{ display: 'flex', alignItems: 'center', flexDirection: 'row', flexWrap: 'wrap' }}>
                      {loading ? (
                        <CircularProgress size={30} />
                      ) : (
                        permissions?.data.map(item => (
                          <FormControlLabel
                            key={item?.id}
                            control={
                              <Checkbox
                                value={item?.id}
                                checked={permissionList.includes(Number.parseInt(item?.id))}
                                onChange={handleChange}
                              />
                            }
                            label={item?.name}
                          />
                        ))
                      )}
                    </FormGroup>
                    <FormHelperText>{errors['permissions']}</FormHelperText>
                  </FormControl>
                </Box>
              )}
            </DialogContentText>
          </DialogContent>
          <DialogActions>
            <Button onClick={handleClose} color={'error'} variant={'outlined'} size={'small'}>
              Close
            </Button>
            <Button
              type={'submit'}
              disabled={buttonLoading}
              color={'success'}
              variant={'outlined'}
              size={'small'}
              endIcon={<SaveIcon className={'h-4'} />}
            >
              {buttonLoading ? <CircularProgress size={20} /> : 'Save'}
            </Button>
          </DialogActions>
        </form>
      </Dialog>
    </div>
  )
}

export default RoleForm
